@@ -451,12 +451,8 @@ dlo_dev_t dlo_claim_device(const dlo_dev_t uid, const dlo_claim_t flags, const u
   /* Any other errors from opening the connection get returned to the caller */
   ERR_GOTO(err);
 
-  /* Select the standard output channel */
-  ERR_GOTO(dlo_usb_std_chan(dev));
-
   /* Attempt to change mode into the native resolution of the display (if we have one) */
-  if (dev->native.view.width)
-    ERR_GOTO(dlo_mode_change(dev, &dev->native, DLO_INVALID_MODE));
+  dlo_mode_set_default(dev, 0);
 
   return uid;
 
@@ -467,6 +463,37 @@ error:
 }
 
 
+static const char* parse_cmdline(int *argc_p, char *argv[])
+{
+  /* Scan the command line looking for --dlo:display.  Return the value found, if
+   * any, else return NULL.  Remove all --dlo:* options from argv.
+   */
+
+  const char *display = NULL;
+  int n;
+  int m = 1;
+
+  if (!argc_p) {
+    return NULL;
+  }
+
+  for (n=1; n<*argc_p; ++n) {
+    const char* arg = argv[n];
+    if (strncmp(arg,"--dlo:",6)==0) {
+      const char* dlo_arg = arg + 6;
+      if (strncmp(dlo_arg,"display=",8)==0) {
+        display = dlo_arg + 8;
+      }
+    } else {
+      argv[m] = arg;
+      ++m;
+    }
+  }
+  *argc_p = m;
+  return display;
+}
+
+
 dlo_dev_t dlo_claim_first_device(const dlo_claim_t flags, const uint32_t timeout)
 {
   dlo_devlist_t *node;
@@ -495,6 +522,45 @@ dlo_dev_t dlo_claim_first_device(const dlo_claim_t flags, const uint32_t timeout
 }
 
 
+dlo_dev_t dlo_claim_default_device(int *argc_p, char *argv[],
+                                   const dlo_claim_t flags, const uint32_t timeout)
+{
+  const char *display = parse_cmdline(argc_p, argv);
+  if (!display) {
+    display = getenv("DLODISPLAY");
+  }
+
+  // For now, assume that display is simply the serial number.  Some thought about
+  // exactly what its format should be is needed.
+  const char *serial = display;
+
+  dlo_devlist_t *node;
+  dlo_devlist_t *next;
+  dlo_dev_t      uid = 0;
+
+  /* Look for a DisplayLink device to connect to - note the first one which is unclaimed */
+  node = dlo_enumerate_devices();
+  
+  // dlo_enumerate_devices allocates memory for each node, which we must free
+  while (node)
+  {
+    dlo_device_t *dev = (dlo_device_t *)node->dev.uid;
+
+    if (!uid && !dev->claimed
+        && (!serial || strcmp(serial,dev->serial)==0))
+    {
+      uid = dlo_claim_device(node->dev.uid, flags, timeout);
+    }
+
+    /* If we haven't claimed a device, move on to the next one */
+    next = node->next;
+    dlo_free(node);
+    node = next;
+  }
+  return uid;
+}
+
+
 dlo_retcode_t dlo_release_device(const dlo_dev_t uid)
 {
   dlo_device_t *dev = (dlo_device_t *)uid;
@@ -528,6 +594,25 @@ dlo_retcode_t dlo_set_mode(const dlo_dev_t uid, const dlo_mode_t * const desc)
   if (!dev)
     return dlo_err_bad_device;
 
+  /* TODO: clean up the cases we use the direct-from-EDID path, when we 
+   * generally clean up (or is that clean out?) the standard mode tables
+   */
+  if ((desc == NULL) || (desc->view.width == 0) ||
+      /* or if width and hight the same as monitor's preferred mode */
+      ((dev->edid.timings[0].pixelClock10KHz) && (desc->view.width == dev->edid.timings[0].hActive) && (desc->view.height == dev->edid.timings[0].vActive))) {
+
+    uint32_t base = 0;
+
+    if (desc) {
+      base = desc->view.base;
+    }
+
+    /* Then do a modeset directly from the preferred mode in the EDID */
+    return dlo_mode_set_default(dev, base);
+  }
+
+  DPRINTF("dlo: set_mode: asking for width %u height %u refresh %u bpp %u\n", desc->view.width, desc->view.height, desc->refresh, desc->view.bpp);
+
   /* See if we can provide a mode which matches the required parameters */
   num = dlo_mode_lookup(dev, desc->view.width, desc->view.height, desc->refresh, desc->view.bpp);
   DPRINTF("dlo: set_mode: lookup %d\n", (int32_t)num);
